package auth

import (
	"context"
	"testing"

	"git.perx.ru/perxis/perxis-go/pkg/environments"
	mocksenvs "git.perx.ru/perxis/perxis-go/pkg/environments/mocks"
	"git.perx.ru/perxis/perxis-go/pkg/roles"
	"github.com/stretchr/testify/mock"
)

func Test_hasEnvironmentAccess(t *testing.T) {
	type args struct {
		ctx      context.Context
		envscall func(envsservice *mocksenvs.Environments)
		role     *roles.Role
		envID    string
	}
	tests := []struct {
		name string
		args args
		want bool
	}{
		{
			name: "simple",
			args: args{
				ctx: context.Background(),
				role: &roles.Role{
					ID:           "1",
					SpaceID:      "space",
					Description:  "Current",
					Environments: []string{"env1", "env2"},
				},
				envID: "env1",
			},
			want: true,
		},
		{
			name: "glob env in role test: e*",
			args: args{
				ctx: context.Background(),
				envscall: func(envsservice *mocksenvs.Environments) {
					envsservice.On("Get", mock.Anything, mock.Anything, mock.Anything).Return(&environments.Environment{
						ID:      "env1",
						SpaceID: "space",
						Aliases: []string{"master"},
					}, nil).Once()
				},
				role: &roles.Role{
					ID:           "1",
					SpaceID:      "space",
					Description:  "Current",
					Environments: []string{"e*"},
				},
				envID: "env",
			},
			want: true,
		},
		{
			name: "glob env in role test: *n*",
			args: args{
				ctx: context.Background(),
				envscall: func(envsservice *mocksenvs.Environments) {
					envsservice.On("Get", mock.Anything, mock.Anything, mock.Anything).Return(&environments.Environment{
						ID:      "env1",
						SpaceID: "space",
						Aliases: []string{"master"},
					}, nil).Once()
				},
				role: &roles.Role{
					ID:           "1",
					SpaceID:      "space",
					Description:  "Current",
					Environments: []string{"*n*"},
				},
				envID: "env",
			},
			want: true,
		},
		{
			name: "glob env in role test: *1",
			args: args{
				ctx: context.Background(),
				envscall: func(envsservice *mocksenvs.Environments) {
					envsservice.On("Get", mock.Anything, mock.Anything, mock.Anything).Return(&environments.Environment{
						ID:      "env1",
						SpaceID: "space",
						Aliases: []string{"master"},
					}, nil).Once()
				},
				role: &roles.Role{
					ID:           "1",
					SpaceID:      "space",
					Description:  "Current",
					Environments: []string{"*1"},
				},
				envID: "env",
			},
			want: true,
		},
		{
			name: "glob env in role test (alias): ma*",
			args: args{
				ctx: context.Background(),
				envscall: func(envsservice *mocksenvs.Environments) {
					envsservice.On("Get", mock.Anything, mock.Anything, mock.Anything).Return(&environments.Environment{
						ID:      "env1",
						SpaceID: "space",
						Aliases: []string{"master"},
					}, nil).Once()
				},
				role: &roles.Role{
					ID:           "1",
					SpaceID:      "space",
					Description:  "Current",
					Environments: []string{"ma*"},
				},
				envID: "env1",
			},
			want: true,
		},
		{
			name: "glob env in role test: *",
			args: args{
				ctx: context.Background(),
				envscall: func(envsservice *mocksenvs.Environments) {
					envsservice.On("Get", mock.Anything, mock.Anything, mock.Anything).Return(&environments.Environment{
						ID:      "env1",
						SpaceID: "space",
						Aliases: []string{"master"},
					}, nil).Once()
				},
				role: &roles.Role{
					ID:           "1",
					SpaceID:      "space",
					Description:  "Current",
					Environments: []string{"*"},
				},
				envID: "env1",
			},
			want: true,
		},
		{
			name: "glob env in role test: q*",
			args: args{
				ctx: context.Background(),
				envscall: func(envsservice *mocksenvs.Environments) {
					envsservice.On("Get", mock.Anything, mock.Anything, mock.Anything).Return(&environments.Environment{
						ID:      "env1",
						SpaceID: "space",
						Aliases: []string{"master"},
					}, nil).Once()
				},
				role: &roles.Role{
					ID:           "1",
					SpaceID:      "space",
					Description:  "Current",
					Environments: []string{"q*"},
				},
				envID: "env1",
			},
			want: false,
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			envsservice := &mocksenvs.Environments{}
			if tt.args.envscall != nil {
				tt.args.envscall(envsservice)
			}

			if got := hasEnvironmentAccess(tt.args.ctx, envsservice, tt.args.role, tt.args.envID); got != tt.want {
				t.Errorf("hasEnvironmentAccess() = %v, want %v", got, tt.want)
			}
		})
	}
}
